@@ -1,13 +1,13 @@
-## ---- include = FALSE----------------------------------------------------
+## ---- include = FALSE---------------------------------------------------------
 knitr::opts_chunk$set(message = FALSE)
 library("cobalt")
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 data("lalonde", package = "cobalt")
 covs <- subset(lalonde, select = -c(treat, re78))
 f.build("treat", covs)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  # Generating propensity scores using logistic regression
 #  p.score <- glm(f.build("treat", covs), data = lalonde, family = "binomial")$fitted.values
 #  
@@ -15,20 +15,20 @@ f.build("treat", covs)
 #  library("MatchIt")
 #  m.out <- matchit(f.build("treat", covs), data = lalonde, method = "nearest")
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 head(lalonde)
 lalonde.split <- splitfactor(lalonde, "race")
 head(lalonde.split)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 lalonde.unsplit <- unsplitfactor(lalonde.split, "race", 
                                  dropped.level = "black")
 head(lalonde.unsplit)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"twang" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ---- warning = FALSE----------------------------------------------------
+## ---- warning = FALSE---------------------------------------------------------
 library("twang")
 data("lalonde", package = "cobalt") ##If not yet loaded
 covs0 <- subset(lalonde, select = -c(treat, re78, nodegree, married))
@@ -38,13 +38,13 @@ ps.out <- ps(f.build("treat", covs0), data = lalonde,
              estimand = "ATT", n.trees = 1000, verbose = FALSE)
 bal.tab(ps.out, stop.method = "es.mean")
 
-## ---- include=FALSE, eval=TRUE-------------------------------------------
+## ---- include=FALSE, eval=TRUE------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"Matching" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("Matching")
 data("lalonde", package = "cobalt") #If not yet loaded
 covs0 <- subset(lalonde, select = -c(treat, re78, nodegree, married))
@@ -55,16 +55,16 @@ match.out <- Match(Tr = lalonde$treat, X = p.score, estimand = "ATT")
 
 bal.tab(match.out, formula = f.build("treat", covs0), data = lalonde)
 
-## ---- eval = FALSE-------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
 #  bal.tab(match.out, treat = lalonde$treat, covs = covs0)
 
-## ---- include=FALSE, eval=TRUE-------------------------------------------
+## ---- include=FALSE, eval=TRUE------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"optmatch" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 #Optimal full matching on the propensity score
 library("optmatch")
 data("lalonde", package = "cobalt") #If not yet loaded
@@ -76,13 +76,13 @@ fm <- fullmatch(treat ~ p.score, data = lalonde)
 
 bal.tab(fm, formula = f.build("treat", covs0), data = lalonde)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"CBPS" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("CBPS")
 data("lalonde", package = "cobalt") #If not yet loaded
 covs0 <- subset(lalonde, select = -c(treat, re78, nodegree, married))
@@ -92,13 +92,13 @@ cbps.out <- CBPS(f.build("treat", covs0), data = lalonde)
 
 bal.tab(cbps.out)
 
-## ---- include=FALSE, eval=TRUE-------------------------------------------
+## ---- include=FALSE, eval=TRUE------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"ebal" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("ebal")
 data("lalonde", package = "cobalt") #If not yet loaded
 covs0 <- subset(lalonde, select = -c(treat, re78, race))
@@ -108,13 +108,13 @@ e.out <- ebalance(lalonde$treat, covs0)
 
 bal.tab(e.out, treat = lalonde$treat, covs = covs0)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 if (!"designmatch" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("designmatch")
 data("lalonde", package = "cobalt") #If not yet loaded
 covs0 <- subset(lalonde, select = -c(treat, re78, race))
@@ -130,13 +130,37 @@ dmout <- bmatch(lalonde$treat,
 
 bal.tab(dmout, treat = lalonde$treat, covs = covs0)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
+if (!"MatchThem" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
+
+## -----------------------------------------------------------------------------
+library("MatchThem"); library("mice")
+data("lalonde_mis", package = "cobalt")
+
+#Generate imputed data sets
+m <- 10 #number of imputed data sets
+imp.out <- mice(lalonde_mis, m = m, print = FALSE) 
+
+#Matching for balance on covariates
+wt.out <- weightthem(treat ~ age + educ + married +
+                       race + re74 + re75, 
+                     datasets = imp.out,
+                     approach = "within", 
+                     method = "ps",
+                     estimand = "ATE")
+
+bal.tab(wt.out)
+
+## ---- include=FALSE-----------------------------------------------------------
+knitr::opts_chunk$set(eval = TRUE)
+
+## ---- include=FALSE-----------------------------------------------------------
 if (!"optweight" %in% rownames(installed.packages())) knitr::opts_chunk$set(eval = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library("optweight")
 data("lalonde", package = "cobalt")
 
@@ -150,6 +174,6 @@ names(ow.out)
 #Use bal.tab() directly on the output
 bal.tab(ow.out)
 
-## ---- include=FALSE------------------------------------------------------
+## ---- include=FALSE-----------------------------------------------------------
 knitr::opts_chunk$set(eval = TRUE)
 
